#include <linux/init.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/fs.h>
MODULE_LICENSE("DUAL BSD/GPL");

//declare our proc file struct
static struct proc_dir_entry* helloworld_file;

//this is the function which actually defines the content of the file. Why? Because we pass it
//on to single_open as a parameter, within helloworld_open, which gets called when our file is opened.
//(see below)
static int helloworld_show(struct seq_file *m, void *v)
{
    seq_printf(m, "Hello world!\n");
    return 0;
}

//this function gets called when we open the proc file
//the second argument of this function is only important when you are sequencing through output
static int helloworld_open(struct inode *inode, struct file *file)
{
    printk(KERN_ALERT "Our proc file was opened!");
    //single_open can be used when we are not sequencing
    return single_open(file, helloworld_show, NULL);
}

//define file operations for our file
static const struct file_operations helloworld_fops = {
     .owner	= THIS_MODULE,
     .open	= helloworld_open, //this is really the only one you need to change for our purposes
     .read	= seq_read,
     .llseek	= seq_lseek,
     .release	= single_release,
 };

static int __init hello_init(void)
{
    //create the proc file
    helloworld_file = proc_create("helloworld", 0, NULL, &helloworld_fops);

    //if we were unable to create it, return an error
    if(!helloworld_file)
    {
        return -ENOMEM; //out of memory error
    }

    return 0;
}

static void __exit hello_exit(void)
{
    //delete the proc file
    remove_proc_entry("helloworld", NULL);
}

module_init(hello_init);
module_exit(hello_exit);
